#!/usr/bin/env python
# -*- coding: utf-8 -*-

__copyright__ = """
   pySART - Simplified AUTOSAR-Toolkit for Python.

   (C) 2010-2020 by Christoph Schueler <cpu12.gems.googlemail.com>

   All Rights Reserved

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

   s. FLOSS-EXCEPTION.txt
"""

import pydbc
from pydbc.types import BusType
import pydbc.db.model as model
from pydbc.db import VNDB

from sqlalchemy import event

@event.listens_for(model.Message.message_signals, 'append', retval = True)
def my_append_listener(target, value, initiator):
    print("target: {} \n\tvalue: {}\n".format(target, value, initiator))
    return value

#event.listen(model.Message.message_signals, 'append', my_append_listener)

DEBUG = False


def main():
    #cdb = VNDB.create("create_candb", debug = DEBUG)
    cdb = VNDB.create(":memory:", debug = DEBUG)
    #cdb = VNDB.open("create_candb", debug = DEBUG)
    session = cdb.session

    engine = model.Node(name = "Engine")
    gateway = model.Node(name = "Gateway")
    session.add_all([engine, gateway])


    engine_data = model.Message(name = "EngineData", message_id = 100, dlc = 8, sender = engine.rid)
    session.add(engine_data)

    petrolLevel = model.Signal(name = "PetrolLevel", bitsize = 8, byteorder = 1,
        sign = +1, formula_factor = 1.0, formula_offset = 0.0, minimum = 0, maximum = 255, unit = "l"
    )
    engPower = model.Signal(name = "EngPower", bitsize = 16, byteorder = 1,
        sign = +1, formula_factor = 0.01, formula_offset = 0.0, minimum = 0, maximum = 150, unit = "kw"
    )
    engForce = model.Signal(name = "EngForce", bitsize = 16, byteorder = 1,
        sign = +1, formula_factor = 1.0, formula_offset = 0.0, minimum = 0, maximum = 0, unit = "N"
    )
    session.add_all([petrolLevel, engPower, engForce])

    ms0 = model.Message_Signal(message = engine_data, signal = petrolLevel, offset = 24)
    ms1 = model.Message_Signal(message = engine_data, signal = engPower, offset = 48)
    ms2 = model.Message_Signal(message = engine_data, signal = engForce, offset = 32)

    #petrolLevel.receiver.append(gateway)
    session.add_all([ms0, ms1, ms2])

    v0 = model.Value_Description(value = 0, value_description = "Running")
    v1 = model.Value_Description(value = 1, value_description = "Idle")
    vtgIdleRunning = model.Valuetable(name = "vtgIdleRunning", values = [v0, v1])
    session.add_all([v0, v1, vtgIdleRunning])

    msgs = session.query(model.Message).all()
    for msg in msgs:
        print(msg.signals)

    session.flush()
    session.commit()

if __name__ == '__main__':
    main()

"""


"""



# Generated from ldf.g4 by ANTLR 4.13.2
from antlr4 import *
if "." in __name__:
    from .ldfParser import ldfParser
else:
    from ldfParser import ldfParser

# This class defines a complete generic visitor for a parse tree produced by ldfParser.

class ldfVisitor(ParseTreeVisitor):

    # Visit a parse tree produced by ldfParser#lin_description_file.
    def visitLin_description_file(self, ctx:ldfParser.Lin_description_fileContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#lin_protocol_version_def.
    def visitLin_protocol_version_def(self, ctx:ldfParser.Lin_protocol_version_defContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#lin_language_version_def.
    def visitLin_language_version_def(self, ctx:ldfParser.Lin_language_version_defContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#lin_file_revision_def.
    def visitLin_file_revision_def(self, ctx:ldfParser.Lin_file_revision_defContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#lin_speed_def.
    def visitLin_speed_def(self, ctx:ldfParser.Lin_speed_defContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#channel_name_def.
    def visitChannel_name_def(self, ctx:ldfParser.Channel_name_defContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#node_def.
    def visitNode_def(self, ctx:ldfParser.Node_defContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#node_attributes_def.
    def visitNode_attributes_def(self, ctx:ldfParser.Node_attributes_defContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#node_attribute.
    def visitNode_attribute(self, ctx:ldfParser.Node_attributeContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#attributes_def.
    def visitAttributes_def(self, ctx:ldfParser.Attributes_defContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#configurable_frames.
    def visitConfigurable_frames(self, ctx:ldfParser.Configurable_framesContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#configurable_frame.
    def visitConfigurable_frame(self, ctx:ldfParser.Configurable_frameContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#node_composition_def.
    def visitNode_composition_def(self, ctx:ldfParser.Node_composition_defContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#configuration.
    def visitConfiguration(self, ctx:ldfParser.ConfigurationContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#configuration_item.
    def visitConfiguration_item(self, ctx:ldfParser.Configuration_itemContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#signal_def.
    def visitSignal_def(self, ctx:ldfParser.Signal_defContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#signal_item.
    def visitSignal_item(self, ctx:ldfParser.Signal_itemContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#init_value.
    def visitInit_value(self, ctx:ldfParser.Init_valueContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#init_value_scalar.
    def visitInit_value_scalar(self, ctx:ldfParser.Init_value_scalarContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#init_value_array.
    def visitInit_value_array(self, ctx:ldfParser.Init_value_arrayContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#diagnostic_signal_def.
    def visitDiagnostic_signal_def(self, ctx:ldfParser.Diagnostic_signal_defContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#diagnostic_item.
    def visitDiagnostic_item(self, ctx:ldfParser.Diagnostic_itemContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#signal_groups_def.
    def visitSignal_groups_def(self, ctx:ldfParser.Signal_groups_defContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#signal_group.
    def visitSignal_group(self, ctx:ldfParser.Signal_groupContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#signal_group_item.
    def visitSignal_group_item(self, ctx:ldfParser.Signal_group_itemContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#frame_def.
    def visitFrame_def(self, ctx:ldfParser.Frame_defContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#frame_item.
    def visitFrame_item(self, ctx:ldfParser.Frame_itemContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#frame_signal.
    def visitFrame_signal(self, ctx:ldfParser.Frame_signalContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#sporadic_frame_def.
    def visitSporadic_frame_def(self, ctx:ldfParser.Sporadic_frame_defContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#sporadic_frame_item.
    def visitSporadic_frame_item(self, ctx:ldfParser.Sporadic_frame_itemContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#event_triggered_frame_def.
    def visitEvent_triggered_frame_def(self, ctx:ldfParser.Event_triggered_frame_defContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#event_triggered_frame_item.
    def visitEvent_triggered_frame_item(self, ctx:ldfParser.Event_triggered_frame_itemContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#diag_frame_def.
    def visitDiag_frame_def(self, ctx:ldfParser.Diag_frame_defContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#diag_frame_item.
    def visitDiag_frame_item(self, ctx:ldfParser.Diag_frame_itemContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#schedule_table_def.
    def visitSchedule_table_def(self, ctx:ldfParser.Schedule_table_defContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#schedule_table_entry.
    def visitSchedule_table_entry(self, ctx:ldfParser.Schedule_table_entryContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#schedule_table_command.
    def visitSchedule_table_command(self, ctx:ldfParser.Schedule_table_commandContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#command.
    def visitCommand(self, ctx:ldfParser.CommandContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#signal_encoding_type_def.
    def visitSignal_encoding_type_def(self, ctx:ldfParser.Signal_encoding_type_defContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#signal_encoding_entry.
    def visitSignal_encoding_entry(self, ctx:ldfParser.Signal_encoding_entryContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#signal_encoding_value.
    def visitSignal_encoding_value(self, ctx:ldfParser.Signal_encoding_valueContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#logical_value.
    def visitLogical_value(self, ctx:ldfParser.Logical_valueContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#physical_range.
    def visitPhysical_range(self, ctx:ldfParser.Physical_rangeContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#bcd_value.
    def visitBcd_value(self, ctx:ldfParser.Bcd_valueContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#ascii_value.
    def visitAscii_value(self, ctx:ldfParser.Ascii_valueContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#signal_representation_def.
    def visitSignal_representation_def(self, ctx:ldfParser.Signal_representation_defContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#signal_representation_entry.
    def visitSignal_representation_entry(self, ctx:ldfParser.Signal_representation_entryContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#intValue.
    def visitIntValue(self, ctx:ldfParser.IntValueContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#floatValue.
    def visitFloatValue(self, ctx:ldfParser.FloatValueContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#number.
    def visitNumber(self, ctx:ldfParser.NumberContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#stringValue.
    def visitStringValue(self, ctx:ldfParser.StringValueContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ldfParser#identifierValue.
    def visitIdentifierValue(self, ctx:ldfParser.IdentifierValueContext):
        return self.visitChildren(ctx)



del ldfParser


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
python-can integration helpers for pyDBC.

This module provides:
- encode_message: Pack physical signal values into a CAN frame payload
  according to the DBC data stored in the VNDB (SQLAlchemy session).
- decode_message: Unpack a CAN frame payload back to physical signal values.
- Optional convenience sender/receiver classes that use python-can if installed.

Notes/assumptions:
- Currently supports little-endian (Intel) signals (Signal.byteorder == 1).
  Big-endian (Motorola) signals raise NotImplementedError.
- Signal.sign: In the current model, sign==1 is treated as unsigned, 0 as signed.
- Physical<->raw conversion uses: phys = raw * factor + offset
  and raw = round((phys - offset) / factor).

This module avoids a hard dependency on python-can. The Bus-related classes
attempt to import python-can at runtime and fail gracefully with clear messages
if it's not installed.
"""
from __future__ import annotations

from dataclasses import dataclass
from typing import Dict, Tuple, Any, Optional, List

from sqlalchemy.orm import Session

from pydbc.db.model import Message, Message_Signal, Signal


@dataclass
class CompiledSignal:
    name: str
    start_bit: int  # bit offset in frame
    size: int       # bits
    little_endian: bool
    signed: bool
    factor: float
    offset: float
    minimum: float
    maximum: float


@dataclass
class CompiledMessage:
    message_id: int
    dlc: int
    name: Optional[str]
    signals: List[CompiledSignal]


def _compile_message(session: Session, message: Message) -> CompiledMessage:
    sigs: List[CompiledSignal] = []
    for ms in message.message_signals:
        sig: Signal = ms.signal
        if sig.byteorder not in (0, 1):
            raise ValueError(f"Unsupported byteorder {sig.byteorder} for signal {sig.name}")
        little = sig.byteorder == 1
        signed = (sig.sign == 0)  # treat 0 as signed, 1 as unsigned
        sigs.append(
            CompiledSignal(
                name=sig.name,
                start_bit=ms.offset,
                size=sig.bitsize,
                little_endian=little,
                signed=signed,
                factor=sig.formula_factor or 1.0,
                offset=sig.formula_offset or 0.0,
                minimum=sig.minimum if sig.minimum is not None else 0.0,
                maximum=sig.maximum if sig.maximum is not None else 0.0,
            )
        )
    return CompiledMessage(
        message_id=message.message_id or 0,
        dlc=message.dlc,
        name=message.name,
        signals=sigs,
    )


def get_message_by_name(session: Session, name: str) -> Message:
    msg = session.query(Message).filter_by(name=name).first()
    if not msg:
        raise KeyError(f"Message with name '{name}' not found")
    return msg


def get_message_by_id(session: Session, can_id: int) -> Message:
    msg = session.query(Message).filter_by(message_id=can_id).first()
    if not msg:
        raise KeyError(f"Message with ID 0x{can_id:X} not found")
    return msg


# --- Bit packing utilities (little-endian/Intel only) ---

def _insert_bits_le(buf: bytearray, start_bit: int, size: int, value: int) -> None:
    """Insert 'size' bits of 'value' into buf at little-endian start_bit.
    Little-endian here means the start_bit 0 is the LSB of byte 0, increasing upwards.
    """
    # Mask value to size bits
    if size <= 0:
        return
    mask = (1 << size) - 1
    value &= mask

    bit_pos = start_bit
    remaining = size
    while remaining > 0:
        byte_index = bit_pos // 8
        bit_index = bit_pos % 8
        bits_in_this_byte = min(remaining, 8 - bit_index)
        # Extract the chunk to write at current position
        chunk_mask = (1 << bits_in_this_byte) - 1
        chunk = value & chunk_mask
        # Clear target bits
        buf[byte_index] &= ~(chunk_mask << bit_index)
        # Write
        buf[byte_index] |= (chunk << bit_index)
        # Advance
        value >>= bits_in_this_byte
        remaining -= bits_in_this_byte
        bit_pos += bits_in_this_byte


def _extract_bits_le(buf: bytes, start_bit: int, size: int, signed: bool) -> int:
    if size <= 0:
        return 0
    bit_pos = start_bit
    remaining = size
    out = 0
    shift = 0
    while remaining > 0:
        byte_index = bit_pos // 8
        bit_index = bit_pos % 8
        bits_in_this_byte = min(remaining, 8 - bit_index)
        chunk_mask = (1 << bits_in_this_byte) - 1
        byte_val = buf[byte_index]
        chunk = (byte_val >> bit_index) & chunk_mask
        out |= (chunk << shift)
        remaining -= bits_in_this_byte
        bit_pos += bits_in_this_byte
        shift += bits_in_this_byte
    if signed:
        # sign-extend
        sign_bit = 1 << (size - 1)
        if out & sign_bit:
            out = out - (1 << size)
    return out


def _phys_to_raw(value: float, factor: float, offset: float, size: int, signed: bool) -> int:
    if factor == 0:
        # Avoid division by zero; treat as identity around offset
        raw = int(round(value - offset))
    else:
        raw = int(round((value - offset) / factor))
    # Clamp to representable range
    if signed:
        min_raw = -(1 << (size - 1))
        max_raw = (1 << (size - 1)) - 1
    else:
        min_raw = 0
        max_raw = (1 << size) - 1
    return max(min(raw, max_raw), min_raw)


def _raw_to_phys(raw: int, factor: float, offset: float) -> float:
    return raw * factor + offset


def encode_message(session: Session, message_name: str, signal_values: Dict[str, float]) -> Tuple[int, bytes, int]:
    """Encode a message by name into (can_id, data_bytes, dlc).

    Returns a tuple suitable for constructing a python-can Message:
    (arbitration_id, data, dlc)
    """
    msg = get_message_by_name(session, message_name)
    cm = _compile_message(session, msg)
    data = bytearray([0] * cm.dlc)

    # Place each signal
    for sig in cm.signals:
        if not sig.little_endian:
            raise NotImplementedError(
                f"Signal '{sig.name}' is big-endian (Motorola); not supported yet."
            )
        if sig.name not in signal_values:
            # Missing values default to 0 after scaling
            phys = 0.0
        else:
            phys = float(signal_values[sig.name])
        raw = _phys_to_raw(phys, sig.factor, sig.offset, sig.size, sig.signed)
        _insert_bits_le(data, sig.start_bit, sig.size, raw)

    return cm.message_id, bytes(data), cm.dlc


def decode_message(session: Session, can_id: int, data: bytes) -> Dict[str, Any]:
    """Decode a received CAN frame given arbitration id and data payload.

    Returns dict: {"message": name, "signals": {sig_name: physical_value, ...}}
    """
    msg = get_message_by_id(session, can_id)
    cm = _compile_message(session, msg)
    result: Dict[str, float] = {}

    for sig in cm.signals:
        if not sig.little_endian:
            raise NotImplementedError(
                f"Signal '{sig.name}' is big-endian (Motorola); not supported yet."
            )
        raw = _extract_bits_le(data, sig.start_bit, sig.size, sig.signed)
        phys = _raw_to_phys(raw, sig.factor, sig.offset)
        result[sig.name] = phys

    return {"message": cm.name or f"0x{cm.message_id:X}", "signals": result}


class PythonCanSender:
    """Simple sender using python-can Bus.

    Usage:
        sender = PythonCanSender(channel='vcan0', bustype='virtual')
        arb_id, data, dlc = encode_message(session, 'EngineData', {...})
        sender.send(arb_id, data)
    """

    def __init__(self, **bus_kwargs):
        try:
            import can  # type: ignore
        except Exception as exc:
            raise RuntimeError(
                "python-can is required for PythonCanSender. Install with 'pip install python-can'."
            ) from exc
        self._can = can
        self._bus = can.Bus(**bus_kwargs)

    def send(self, arbitration_id: int, data: bytes, is_extended_id: bool = False) -> None:
        msg = self._can.Message(
            arbitration_id=arbitration_id,
            is_extended_id=is_extended_id,
            data=data,
        )
        self._bus.send(msg)

    def shutdown(self) -> None:
        try:
            self._bus.shutdown()
        except Exception:
            pass


class PythonCanReceiver:
    """Simple receiver that decodes messages with a pydbc session.

    Example:
        rx = PythonCanReceiver(session, channel='vcan0', bustype='virtual')
        frame = rx.recv(timeout=1.0)
        if frame: print(rx.decode_frame(frame))
    """

    def __init__(self, session: Session, **bus_kwargs):
        try:
            import can  # type: ignore
        except Exception as exc:
            raise RuntimeError(
                "python-can is required for PythonCanReceiver. Install with 'pip install python-can'."
            ) from exc
        self._can = can
        self._bus = can.Bus(**bus_kwargs)
        self._session = session

    def recv(self, timeout: Optional[float] = None):
        return self._bus.recv(timeout=timeout)

    def decode_frame(self, frame) -> Dict[str, Any]:
        return decode_message(self._session, frame.arbitration_id, bytes(frame.data))

    def shutdown(self) -> None:
        try:
            self._bus.shutdown()
        except Exception:
            pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__copyright__ = """
   pySART - Simplified AUTOSAR-Toolkit for Python.

   (C) 2010-2025 by Christoph Schueler <cpu12.gems.googlemail.com>

   All Rights Reserved

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

   s. FLOSS-EXCEPTION.txt
"""
__author__ = "Christoph Schueler"
__version__ = "0.1.0"

import codecs
import importlib
import os
from pprint import pprint
import sys

import antlr4
import antlr4.tree

from sqlalchemy.sql.expression import literal, bindparam
from sqlalchemy.ext import baked

from pydbc.logger import Logger
from pydbc.db import VNDB
from pydbc.db.model import (
    Dbc_Version,
    Message,
    Message_Signal,
    Network,
    Node,
    Signal,
    Value_Description,
    Valuetable,
    EnvironmentVariablesData,
    EnvVar,
    Attribute_Definition,
    Attribute_Value,
    Node_TxMessage,
    Node_RxSignal,
    Category_Definition,
    Category_Value,
    AttributeRel_Value,
    Signal_Group_Signal,
    Signal_Group,
    Vndb_Protocol,
    Object_Valuetable,
)
from pydbc.utils import detect_encoding


def indent(level):
    print(
        " " * level,
    )


def dump(tree, level=0):
    indent(level)
    if isinstance(tree, antlr4.TerminalNode):
        print(tree.symbol.text)
    else:
        print("({}".format(tree.value))
        level += 1
        for child in tree.children:
            dump(child, level)
        level -= 1
    indent(level)
    print(")")


def toInt(number, base=10):
    """ """
    try:
        res = int(number, base)
    except ValueError as e:
        res = None
    return res


def toFloat(number):
    """ """
    try:
        res = float(number)
    except ValueError as e:
        res = None
    return res


class BaseListener(antlr4.ParseTreeListener):
    """ """

    value = []

    def __init__(self, database, logLevel="INFO"):
        self.db = database
        self.logger = Logger(__name__, level=logLevel)
        super(BaseListener, self).__init__()
        self.session = database.session
        self.bakery = baked.bakery()
        self.bake_common_queries()

    def bake_common_queries(self):
        self.ATTRIBUTE_DEFINITION_BY_NAME = self.bakery(
            lambda session: self.session.query(Attribute_Definition).filter(
                Attribute_Definition.name == bindparam("name")
            )
        )
        self.MESSAGE_BY_NAME = self.bakery(
            lambda session: session.query(Message).filter(
                Message.name == bindparam("name")
            )
        )
        self.NODE_BY_NAME = self.bakery(
            lambda session: session.query(Node).filter(Node.name == bindparam("name"))
        )
        self.NODE_BY_RID = self.bakery(
            lambda session: session.query(Node).filter(Node.rid == bindparam("rid"))
        )
        self.SIGNAL_BY_NAME = self.bakery(
            lambda session: session.query(Signal).filter(
                Signal.name == bindparam("name")
            )
        )
        self.SIGNAL_BY_RID = self.bakery(
            lambda session: session.query(Signal).filter(Signal.rid == bindparam("rid"))
        )

    def log_insertion(self, table_name):
        self.logger.debug("Inserting values for '{}'.".format(table_name))

    def getList(self, attr):
        return [x for x in attr] if attr else []

    def getTerminal(self, attr, default=None):
        return attr.text if attr else default

    def getValue(self, attr, default=None):
        return attr.value if attr else default

    def exitIntValue(self, ctx):
        if ctx.i:
            ctx.value = toInt(ctx.i.text, 10)
        elif ctx.h:
            ctx.value = toInt(ctx.h.text, 16)
        else:
            ctx.value = None

    def exitFloatValue(self, ctx):
        ctx.value = toFloat(ctx.f.text) if ctx.f else None

    def exitNumber(self, ctx):
        if ctx.i:
            value = ctx.i.value
        elif ctx.f:
            value = ctx.f.value
        else:
            value = None
        ctx.value = value

    def exitStringValue(self, ctx):
        value = ctx.s.text.strip('"') if ctx.s else None
        ctx.value = value

    def exitIdentifierValue(self, ctx):
        value = ctx.i.text if ctx.i else None
        value = None if value == "<missing C_IDENTIFIER>" else value
        ctx.value = value

    def _formatMessage(self, msg, location):
        return "[{0}:{1}] {2}".format(
            location.start.line, location.start.column + 1, msg
        )

    def _log(self, method, msg, location=None):
        if location:
            method(self._formatMessage(msg, location))
        else:
            method(msg)

    def info(self, msg, location=None):
        """Log an info message.

        Args:
            msg: The message to log
            location: Optional location information
        """
        self._log(self.logger.info, msg, location)

    def warn(self, msg, location=None):
        """Log a warning message.

        Args:
            msg: The message to log
            location: Optional location information
        """
        self._log(self.logger.warning, msg, location)

    def error(self, msg, location=None):
        """Log an error message.

        Args:
            msg: The message to log
            location: Optional location information
        """
        self._log(self.logger.error, msg, location)

    def debug(self, msg, location=None):
        """Log a debug message.

        Args:
            msg: The message to log
            location: Optional location information
        """
        self._log(self.logger.debug, msg, location)


class ParserWrapper(object):
    """Wrapper for ANTLR4 parsers.

    This class provides a convenient interface for parsing files using ANTLR4 grammars.
    It dynamically loads the appropriate lexer and parser classes based on the grammar name.
    """

    def __init__(
            self,
            grammarName: str,
            startSymbol: str,
            listenerClass: type,
            debug: bool = False,
            logLevel: str = "INFO",
    ):
        """Initialize the parser wrapper.

        Args:
            grammarName: Name of the grammar (e.g., 'dbc', 'ldf', 'ncf')
            startSymbol: Name of the start rule in the grammar
            listenerClass: Listener class to use for parsing
            debug: Enable debug output
            logLevel: Logging level (INFO, WARN, ERROR, DEBUG)
        """
        self.grammarName = grammarName
        self.startSymbol = startSymbol
        self.lexerModule, self.lexerClass = self._load("Lexer")
        self.parserModule, self.parserClass = self._load("Parser")
        self.listenerClass = listenerClass
        self.debug = debug
        self.logLevel = logLevel
        self._syntaxErrors = 0
        self.db = None
        self.fnbase = None

    def __del__(self):
        """Clean up resources when the object is garbage collected."""
        # Nothing to clean up currently, but this is a placeholder for future cleanup
        pass

    def _load(self, name: str) -> tuple:
        """Load a module and class dynamically.

        Args:
            name: Name suffix for the module and class ('Lexer' or 'Parser')

        Returns:
            Tuple of (module, class)
        """
        className = f"{self.grammarName}{name}"
        moduleName = f"pydbc.py3.{className}"  # Always use Python 3 modules
        module = importlib.import_module(moduleName)
        klass = getattr(module, className)
        return (module, klass)

    def parse(self, input: antlr4.InputStream, trace: bool = False) -> object:
        """Parse input using the configured grammar and listener.

        Args:
            input: ANTLR4 input stream
            trace: Enable trace output for debugging

        Returns:
            SQLAlchemy session object
        """
        self.db = VNDB(self.fnbase, debug=self.debug)
        lexer = self.lexerClass(input)
        tokenStream = antlr4.CommonTokenStream(lexer)
        parser = self.parserClass(tokenStream)
        parser.setTrace(trace)
        meth = getattr(parser, self.startSymbol)
        self._syntaxErrors = parser._syntaxErrors
        tree = meth()
        if self.listenerClass:
            listener = self.listenerClass(self.db, self.logLevel)
            walker = antlr4.ParseTreeWalker()
            walker.walk(listener, tree)
        self.db.session.commit()
        return self.db.session

    def parseFromFile(
            self, filename: str, encoding: str = "ISO-8859-1", trace: bool = False
    ) -> object:  # str = 'latin-1'
        """Parse a file using the configured grammar and listener.

        Args:
            filename: Path to the file to parse
            encoding: Character encoding of the file (auto-detected if possible)
            trace: Enable trace output for debugging

        Returns:
            SQLAlchemy session object
        """
        pth, fname = os.path.split(filename)
        self.fnbase = os.path.splitext(fname)[0]
        return self.parse(ParserWrapper.stringStream(filename, encoding), trace)

    def parseFromString(
            self,
            buf: str,
            encoding: str = "ISO-8859-1",
            trace: bool = False,  # str = 'latin-1'
            dbname: str = ":memory:",
    ) -> object:
        """Parse a string using the configured grammar and listener.

        Args:
            buf: String to parse
            encoding: Character encoding of the string
            trace: Enable trace output for debugging
            dbname: Name of the database to create

        Returns:
            SQLAlchemy session object
        """
        self.fnbase = dbname
        return self.parse(antlr4.InputStream(buf), trace)

    @staticmethod
    def stringStream(fname: str, encoding: str = "ISO-8859-1") -> antlr4.InputStream:
        """Create an ANTLR4 input stream from a file.

        Args:
            fname: Path to the file
            encoding: Character encoding of the file (auto-detected if possible)

        Returns:
            ANTLR4 input stream
        """
        detected_encoding = detect_encoding(fname)
        print(f"Detected encoding: {detected_encoding}")
        # detected_encoding = encoding
        with codecs.open(fname, encoding=detected_encoding) as f:
            return antlr4.InputStream(f.read())

    def _getNumberOfSyntaxErrors(self) -> int:
        """Get the number of syntax errors encountered during parsing.

        Returns:
            Number of syntax errors
        """
        return self._syntaxErrors

    numberOfSyntaxErrors = property(_getNumberOfSyntaxErrors)



# Generated from ncf.g4 by ANTLR 4.13.2
from antlr4 import *
if "." in __name__:
    from .ncfParser import ncfParser
else:
    from ncfParser import ncfParser

# This class defines a complete generic visitor for a parse tree produced by ncfParser.

class ncfVisitor(ParseTreeVisitor):

    # Visit a parse tree produced by ncfParser#toplevel.
    def visitToplevel(self, ctx:ncfParser.ToplevelContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ncfParser#language_version.
    def visitLanguage_version(self, ctx:ncfParser.Language_versionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ncfParser#node_definition.
    def visitNode_definition(self, ctx:ncfParser.Node_definitionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ncfParser#node_name.
    def visitNode_name(self, ctx:ncfParser.Node_nameContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ncfParser#general_definition.
    def visitGeneral_definition(self, ctx:ncfParser.General_definitionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ncfParser#protocol_version.
    def visitProtocol_version(self, ctx:ncfParser.Protocol_versionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ncfParser#supplier_id.
    def visitSupplier_id(self, ctx:ncfParser.Supplier_idContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ncfParser#function_id.
    def visitFunction_id(self, ctx:ncfParser.Function_idContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ncfParser#variant_id.
    def visitVariant_id(self, ctx:ncfParser.Variant_idContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ncfParser#bitrate_definition.
    def visitBitrate_definition(self, ctx:ncfParser.Bitrate_definitionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ncfParser#bitrate.
    def visitBitrate(self, ctx:ncfParser.BitrateContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ncfParser#diagnostic_definition.
    def visitDiagnostic_definition(self, ctx:ncfParser.Diagnostic_definitionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ncfParser#frame_definition.
    def visitFrame_definition(self, ctx:ncfParser.Frame_definitionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ncfParser#single_frame.
    def visitSingle_frame(self, ctx:ncfParser.Single_frameContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ncfParser#frame_kind.
    def visitFrame_kind(self, ctx:ncfParser.Frame_kindContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ncfParser#frame_name.
    def visitFrame_name(self, ctx:ncfParser.Frame_nameContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ncfParser#frame_properties.
    def visitFrame_properties(self, ctx:ncfParser.Frame_propertiesContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ncfParser#signal_definition.
    def visitSignal_definition(self, ctx:ncfParser.Signal_definitionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ncfParser#signal_definition_entry.
    def visitSignal_definition_entry(self, ctx:ncfParser.Signal_definition_entryContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ncfParser#signal_name.
    def visitSignal_name(self, ctx:ncfParser.Signal_nameContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ncfParser#signal_properties.
    def visitSignal_properties(self, ctx:ncfParser.Signal_propertiesContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ncfParser#init_value.
    def visitInit_value(self, ctx:ncfParser.Init_valueContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ncfParser#init_value_scalar.
    def visitInit_value_scalar(self, ctx:ncfParser.Init_value_scalarContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ncfParser#init_value_array.
    def visitInit_value_array(self, ctx:ncfParser.Init_value_arrayContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ncfParser#encoding_definition.
    def visitEncoding_definition(self, ctx:ncfParser.Encoding_definitionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ncfParser#encoding_definition_entry.
    def visitEncoding_definition_entry(self, ctx:ncfParser.Encoding_definition_entryContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ncfParser#encoding_definition_value.
    def visitEncoding_definition_value(self, ctx:ncfParser.Encoding_definition_valueContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ncfParser#encoding_name.
    def visitEncoding_name(self, ctx:ncfParser.Encoding_nameContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ncfParser#logical_value.
    def visitLogical_value(self, ctx:ncfParser.Logical_valueContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ncfParser#physical_range.
    def visitPhysical_range(self, ctx:ncfParser.Physical_rangeContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ncfParser#bcd_value.
    def visitBcd_value(self, ctx:ncfParser.Bcd_valueContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ncfParser#ascii_value.
    def visitAscii_value(self, ctx:ncfParser.Ascii_valueContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ncfParser#signal_value.
    def visitSignal_value(self, ctx:ncfParser.Signal_valueContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ncfParser#min_value.
    def visitMin_value(self, ctx:ncfParser.Min_valueContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ncfParser#max_value.
    def visitMax_value(self, ctx:ncfParser.Max_valueContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ncfParser#scale.
    def visitScale(self, ctx:ncfParser.ScaleContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ncfParser#offset.
    def visitOffset(self, ctx:ncfParser.OffsetContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ncfParser#text_info.
    def visitText_info(self, ctx:ncfParser.Text_infoContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ncfParser#status_management.
    def visitStatus_management(self, ctx:ncfParser.Status_managementContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ncfParser#published_signal.
    def visitPublished_signal(self, ctx:ncfParser.Published_signalContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ncfParser#free_text_definition.
    def visitFree_text_definition(self, ctx:ncfParser.Free_text_definitionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ncfParser#intValue.
    def visitIntValue(self, ctx:ncfParser.IntValueContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ncfParser#floatValue.
    def visitFloatValue(self, ctx:ncfParser.FloatValueContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ncfParser#number.
    def visitNumber(self, ctx:ncfParser.NumberContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ncfParser#stringValue.
    def visitStringValue(self, ctx:ncfParser.StringValueContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by ncfParser#identifierValue.
    def visitIdentifierValue(self, ctx:ncfParser.IdentifierValueContext):
        return self.visitChildren(ctx)



del ncfParser


# Generated from dbc.g4 by ANTLR 4.13.2
from antlr4 import *
if "." in __name__:
    from .dbcParser import dbcParser
else:
    from dbcParser import dbcParser

# This class defines a complete generic visitor for a parse tree produced by dbcParser.

class dbcVisitor(ParseTreeVisitor):

    # Visit a parse tree produced by dbcParser#dbcfile.
    def visitDbcfile(self, ctx:dbcParser.DbcfileContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#messageTransmitters.
    def visitMessageTransmitters(self, ctx:dbcParser.MessageTransmittersContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#messageTransmitter.
    def visitMessageTransmitter(self, ctx:dbcParser.MessageTransmitterContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#signalExtendedValueTypeList.
    def visitSignalExtendedValueTypeList(self, ctx:dbcParser.SignalExtendedValueTypeListContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#signalExtendedValueType.
    def visitSignalExtendedValueType(self, ctx:dbcParser.SignalExtendedValueTypeContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#messages.
    def visitMessages(self, ctx:dbcParser.MessagesContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#message.
    def visitMessage(self, ctx:dbcParser.MessageContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#signal.
    def visitSignal(self, ctx:dbcParser.SignalContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#receiver.
    def visitReceiver(self, ctx:dbcParser.ReceiverContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#transmitter.
    def visitTransmitter(self, ctx:dbcParser.TransmitterContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#multiplexerIndicator.
    def visitMultiplexerIndicator(self, ctx:dbcParser.MultiplexerIndicatorContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#valueTables.
    def visitValueTables(self, ctx:dbcParser.ValueTablesContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#valueTable.
    def visitValueTable(self, ctx:dbcParser.ValueTableContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#valueDescription.
    def visitValueDescription(self, ctx:dbcParser.ValueDescriptionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#nodes.
    def visitNodes(self, ctx:dbcParser.NodesContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#bitTiming.
    def visitBitTiming(self, ctx:dbcParser.BitTimingContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#newSymbols.
    def visitNewSymbols(self, ctx:dbcParser.NewSymbolsContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#version.
    def visitVersion(self, ctx:dbcParser.VersionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#objectValueTables.
    def visitObjectValueTables(self, ctx:dbcParser.ObjectValueTablesContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#objectValueTable.
    def visitObjectValueTable(self, ctx:dbcParser.ObjectValueTableContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#environmentVariables.
    def visitEnvironmentVariables(self, ctx:dbcParser.EnvironmentVariablesContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#environmentVariable.
    def visitEnvironmentVariable(self, ctx:dbcParser.EnvironmentVariableContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#accessNodes.
    def visitAccessNodes(self, ctx:dbcParser.AccessNodesContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#environmentVariablesData.
    def visitEnvironmentVariablesData(self, ctx:dbcParser.EnvironmentVariablesDataContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#environmentVariableData.
    def visitEnvironmentVariableData(self, ctx:dbcParser.EnvironmentVariableDataContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#signalTypes.
    def visitSignalTypes(self, ctx:dbcParser.SignalTypesContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#signalType.
    def visitSignalType(self, ctx:dbcParser.SignalTypeContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#comments.
    def visitComments(self, ctx:dbcParser.CommentsContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#comment.
    def visitComment(self, ctx:dbcParser.CommentContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#attributeDefinitions.
    def visitAttributeDefinitions(self, ctx:dbcParser.AttributeDefinitionsContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#attributeDefinition.
    def visitAttributeDefinition(self, ctx:dbcParser.AttributeDefinitionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#relativeAttributeDefinitions.
    def visitRelativeAttributeDefinitions(self, ctx:dbcParser.RelativeAttributeDefinitionsContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#relativeAttributeDefinition.
    def visitRelativeAttributeDefinition(self, ctx:dbcParser.RelativeAttributeDefinitionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#attributeValueType.
    def visitAttributeValueType(self, ctx:dbcParser.AttributeValueTypeContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#attributeDefaults.
    def visitAttributeDefaults(self, ctx:dbcParser.AttributeDefaultsContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#attributeDefault.
    def visitAttributeDefault(self, ctx:dbcParser.AttributeDefaultContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#relativeAttributeDefaults.
    def visitRelativeAttributeDefaults(self, ctx:dbcParser.RelativeAttributeDefaultsContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#relativeAttributeDefault.
    def visitRelativeAttributeDefault(self, ctx:dbcParser.RelativeAttributeDefaultContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#attributeValue.
    def visitAttributeValue(self, ctx:dbcParser.AttributeValueContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#attributeValues.
    def visitAttributeValues(self, ctx:dbcParser.AttributeValuesContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#attributeValueForObject.
    def visitAttributeValueForObject(self, ctx:dbcParser.AttributeValueForObjectContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#relativeAttributeValues.
    def visitRelativeAttributeValues(self, ctx:dbcParser.RelativeAttributeValuesContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#relativeAttributeValueForObject.
    def visitRelativeAttributeValueForObject(self, ctx:dbcParser.RelativeAttributeValueForObjectContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#signalGroups.
    def visitSignalGroups(self, ctx:dbcParser.SignalGroupsContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#signalGroup.
    def visitSignalGroup(self, ctx:dbcParser.SignalGroupContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#categoryDefinitions.
    def visitCategoryDefinitions(self, ctx:dbcParser.CategoryDefinitionsContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#categoryDefinition.
    def visitCategoryDefinition(self, ctx:dbcParser.CategoryDefinitionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#categories.
    def visitCategories(self, ctx:dbcParser.CategoriesContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#category.
    def visitCategory(self, ctx:dbcParser.CategoryContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#intValue.
    def visitIntValue(self, ctx:dbcParser.IntValueContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#floatValue.
    def visitFloatValue(self, ctx:dbcParser.FloatValueContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#number.
    def visitNumber(self, ctx:dbcParser.NumberContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#stringValue.
    def visitStringValue(self, ctx:dbcParser.StringValueContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by dbcParser#identifierValue.
    def visitIdentifierValue(self, ctx:dbcParser.IdentifierValueContext):
        return self.visitChildren(ctx)



del dbcParser


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Code generators for standalone applications using Mako templates.

This module provides two generators:
- MicroPython CAN application (Python script) using machine.CAN or pyb.CAN depending on platform.
- Linux SocketCAN C source file with simple encode/decode and send/recv demo.

Both generators work from an SQLAlchemy session populated by pydbc (either
created via DBCCreator or returned by ParserWrapper/DbcListener). No runtime
DB access is required by the generated code; all message/signal metadata is
embedded at generation time.
"""
from __future__ import annotations

from dataclasses import dataclass
from typing import List, Optional, Dict, Any, Iterable
import pkgutil

from sqlalchemy.orm import Session

from pydbc.db.model import Message, Message_Signal, Signal
from pydbc.template import renderTemplateFromText


@dataclass
class CGSignal:
    name: str
    start_bit: int
    size: int
    little_endian: bool
    signed: bool
    factor: float
    offset: float
    minimum: float
    maximum: float


@dataclass
class CGMessage:
    name: Optional[str]
    message_id: int
    dlc: int
    signals: List[CGSignal]


def _collect_messages(session: Session, only: Optional[Iterable[str]] = None) -> List[CGMessage]:
    q = session.query(Message)
    if only:
        q = q.filter(Message.name.in_(list(only)))
    msgs: List[CGMessage] = []
    for m in q.all():
        sigs: List[CGSignal] = []
        for ms in m.message_signals:
            s: Signal = ms.signal
            sigs.append(
                CGSignal(
                    name=s.name,
                    start_bit=ms.offset,
                    size=s.bitsize,
                    little_endian=True if (s.byteorder == 1) else False,
                    signed=(s.sign == 0),  # 0=signed, 1=unsigned in current model
                    factor=s.formula_factor or 1.0,
                    offset=s.formula_offset or 0.0,
                    minimum=float(s.minimum) if s.minimum is not None else 0.0,
                    maximum=float(s.maximum) if s.maximum is not None else 0.0,
                )
            )
        msgs.append(
            CGMessage(
                name=m.name,
                message_id=m.message_id or 0,
                dlc=m.dlc,
                signals=sigs,
            )
        )
    # Sort for stable output
    msgs.sort(key=lambda x: (x.message_id, x.name or ""))
    for msg in msgs:
        msg.signals.sort(key=lambda s: s.start_bit)
    return msgs


class MicroPythonCanAppGenerator:
    """Generates a standalone MicroPython CAN app (.py) with encode/decode helpers.

    Usage:
        gen = MicroPythonCanAppGenerator(session)
        code = gen.render(only=["EngineData"])  # optional filter
        Path("micropython_app.py").write_text(code, encoding="utf-8")
    """

    TEMPLATE = pkgutil.get_data("pydbc", "cgen/templates/micropython_can.py.tmpl")

    def __init__(self, session: Session):
        self._session = session

    def render(self, only: Optional[Iterable[str]] = None, app_name: str = "pydbc_mpy_app") -> str:
        msgs = _collect_messages(self._session, only)
        ns: Dict[str, Any] = {
            "app_name": app_name,
            "messages": msgs,
        }
        text = self.TEMPLATE.decode("utf-8") if isinstance(self.TEMPLATE, (bytes, bytearray)) else str(self.TEMPLATE)
        return renderTemplateFromText(text, ns, encoding="utf-8")


class SocketCanCGenerator:
    """Generates a standalone Linux SocketCAN C program (.c) with encode/decode helpers.

    Usage:
        gen = SocketCanCGenerator(session)
        code = gen.render(only=["EngineData"])  # optional filter
        Path("socketcan_app.c").write_text(code, encoding="utf-8")
    """

    TEMPLATE = pkgutil.get_data("pydbc", "cgen/templates/socketcan.c.tmpl")

    def __init__(self, session: Session):
        self._session = session

    def render(self, only: Optional[Iterable[str]] = None, program_name: str = "pydbc_socketcan_app") -> str:
        msgs = _collect_messages(self._session, only)
        ns: Dict[str, Any] = {
            "program_name": program_name,
            "messages": msgs,
        }
        text = self.TEMPLATE.decode("utf-8") if isinstance(self.TEMPLATE, (bytes, bytearray)) else str(self.TEMPLATE)
        return renderTemplateFromText(text, ns, encoding="utf-8")


class PythonCanAppGenerator:
    """Generates eine standalone Python-Anwendung (.py) auf Basis von python-can.

    Nutzung:
        gen = PythonCanAppGenerator(session)
        code = gen.render(only=["EngineData"], app_name="pydbc_python_can_app")
        Path("python_can_app.py").write_text(code, encoding="utf-8")
    """

    TEMPLATE = pkgutil.get_data("pydbc", "cgen/templates/python_can.py.tmpl")

    def __init__(self, session: Session):
        self._session = session

    def render(self, only: Optional[Iterable[str]] = None, app_name: str = "pydbc_python_can_app") -> str:
        msgs = _collect_messages(self._session, only)
        ns: Dict[str, Any] = {
            "app_name": app_name,
            "messages": msgs,
        }
        text = self.TEMPLATE.decode("utf-8") if isinstance(self.TEMPLATE, (bytes, bytearray)) else str(self.TEMPLATE)
        return renderTemplateFromText(text, ns, encoding="utf-8")
